# -*- coding: utf-8 -*-
"""
Module implementing MainWindow.
"""
from __future__ import unicode_literals
import os
import shutil
import platform
from PyQt5.QtCore import pyqtSlot, QSettings, QSize, QPoint
from PyQt5.QtWidgets import QMainWindow, QMessageBox, QFileDialog, QDialog, QApplication
from PyQt5 import QtGui
import youtube_dl
from .Ui_hauptfenster import Ui_MainWindow


class MyLogger(object):
    """
   Diese Klasse speichert die Meldungen von youtube_dl
   und gibt sie aus.
    """
    def __init__(self):
        """
        Dictionary erzeugen, das die Meldungen speichert
        """
        self._message_queue = []
    def debug(self, msg):
        """
        debug-Ereignis
        """
        if msg != '':
            # zum Dictionary hinzufügen und auf die Konsole ausgeben
            self._message_queue.append(msg)
            print('Debug: ' + msg)

    def warning(self, msg):
        """
        Warnmeldungen
        """
        if msg != '':
            self._message_queue.append(msg)
            print('Warning:' + msg)

    def error(self, msg):
        """
        Fehlermeldungen
        """
        if msg != '':
            self._message_queue.append(msg)
            print('Error: ' + msg)
            
    def get_message(self):
        """
        Wenn message_queue etwas enthält, den Inhalt zurückliefern.
        Andernfalls ist der Inhalt None.
        """
        return None if len(self._message_queue) == 0 else self._message_queue

class MainWindow(QMainWindow, Ui_MainWindow):
    def __init__(self, parent=None):
        """
        Constructor
        @param parent reference to the parent widget
        @type QWidget
        
        Hauptfenster erzeugen
        """
        super(MainWindow, self).__init__(parent)
        # Script-Verzeichnis ermitteln
        # Standardmäßig: Youtube-Downloader/ui
        script_dir = os.path.dirname(os.path.realpath(__file__))
        # Icon für die Anwendung festelegen
        self.setWindowIcon(QtGui.QIcon(script_dir + os.path.sep + 'python-256.png'))
        self.setupUi(self)
        # Automatisches scrollen für logtxt aktivieren
        self.logtxt.ensureCursorVisible()
        self.logtxt.setCenterOnScroll(True)
        # Definitionen für das automatische Speichern der Einstellungen
        # Bei Linux wird der Ordner "./config" im Home verwenden
        self.settings = QSettings(
            QSettings.NativeFormat, QSettings.UserScope, "LinuxWelt", "Youtube-Downloader"
            )
        self.settings.beginGroup("MainWindow")
        # Fenstergröße einlesen
        self.resize(self.settings.value("size", QSize(574, 408)))
        # Fensterposition einlesen
        self.move(self.settings.value("pos", QPoint(50, 50)))
        # Weitere Einstellungen
        self.edtPath.setText(self.settings.value("DL-URL", "~"))
        check_state = self.settings.value("keep", False, type=bool)
        self.chkKeep.setChecked(check_state)
        check_state = self.settings.value("ignore", True, type=bool)
        self.chkIgnore.setChecked(check_state)
        self.settings.endGroup()
        # "OK" ist die Standardschaltfläche und wird mit "Enter" ausgelöst
        self.btnOK.setDefault(True)

    def closeEvent(self, event):
        """
        Einstellungen beim Schließen speichern
        """
        self.settings = QSettings(
            QSettings.NativeFormat, QSettings.UserScope, "LinuxWelt", "Youtube-Downloader"
            )
        self.settings.beginGroup("MainWindow")
        self.settings.setValue('size', self.size())
        self.settings.setValue("pos", self.pos())
        self.settings.setValue("DL-URL", self.edtPath.text())
        self.settings.setValue("keep", self.chkKeep.isChecked())
        self.settings.setValue("ignore", self.chkIgnore.isChecked())
        self.settings.endGroup()
        self.settings.sync()

    def y_download(self, dl_url):
        """
        Datei mit youtube_dl herunterladen
        """
        # Download-Pfad aus dem Eingabefeld lesen
        download_path = os.path.expanduser(self.edtPath.text())
        ffmpeg_path = ""
        print("Ihr System:" + platform.system() )
        
        # Pfad zu ffmpeg je nach Plattform
        if platform.system() == "Linux":
            # ffmpeg-Pfad mit which ermitteln
            ffmpeg_path = os.path.dirname(shutil.which('ffmpeg'))
            if ffmpeg_path is None:
                print("Fehler: ffmpeg nicht gefunden")                
                self.logtxt.appendPlainText("Fehler: ffmpeg nicht gefunden")
        else:
            # Windows: ffmpeg aus dem Verzeichnis "ui" verwenden
            ffmpeg_path = os.path.dirname(os.path.abspath(__file__))
                
        if not ffmpeg_path is None:
            print("Verwende " + ffmpeg_path + " als Pfad zu ffmpeg.")
            self.logtxt.appendPlainText("Verwende " + ffmpeg_path + " als Pfad zu ffmpeg.")
        # Die logger-Klasse für youtube_dl
        loggr = MyLogger()
        # Standardwerter
        ignoreerrors = False
        keepvideo = False
        # Optionen festlegen
        if self.chkIgnore.isChecked():
            ignoreerrors = True
        if self.chkKeep.isChecked():
            keepvideo = True
        # Optionen siehe https://github.com/ytdl-org/youtube-dl/blob/master/youtube_dl/YoutubeDL.py#L121-L269            
        ydl_opts = {"logger":  loggr,
                    "outtmpl": os.path.join(download_path, "%(title)s-%(id)s.%(ext)s"),
                    "progress_hooks": [self.my_hook],
                    "ignoreerrors":  ignoreerrors,
                    "keepvideo":keepvideo,
                    "ffmpeg_location":ffmpeg_path,
                    }
        # btnOK deaktivieren
        self.btnOK.setEnabled(False)
        # Hier erfolgt der Download
        with youtube_dl.YoutubeDL(ydl_opts) as ydl:
            ydl.download([dl_url])
        # btnOK wieder aktivieren
        self.btnOK.setEnabled(True)
        # Fehlermeldungen einlesen
        possible_error = loggr.get_message()
        already_downloaded = False
        # Fehlermeldungen ausgeben
        if not possible_error is None:
            for x in possible_error:
                if not already_downloaded and x.find('has already been downloaded') != -1:
                    already_downloaded = True
                if not already_downloaded:
                    if x.find('[download]') == -1:
                        self.logtxt.appendPlainText(x)
        # Fehlermeldung ausgeben, wenn die Datei bereits heruntergeladen wurde
        if already_downloaded:
            self.logtxt.appendPlainText('Fehler: Die Datei wurde bereits heruntergeladen.')

    def my_hook(self, d):
        """
        Meldungen von youtube_dl auslesen
        """
        # Der Download wurde beendet
        if d['status'] == 'finished':
            print('Done downloading, now converting ...')
            self.logtxt.appendPlainText('Done downloading, now converting ...')
            self.progressBar.setValue(0)
            self.btnOK.setEnabled(True)
        # Eine Datei wird heruntergeladen
        if d['status'] == 'downloading':
            logMsg = d['filename'] + '' +  d['_percent_str'] + ' ' + d['_eta_str']
            self.logtxt.appendPlainText(logMsg)
            self.logtxt.ensureCursorVisible()
            # progressBar steuern
            p = d['_percent_str']
            p = p.replace('%', '')
            self.progressBar.setValue(float(p))
            QApplication.processEvents()
        # Fehlermeldungen    
        if d['status'] == 'error':
            print('Error:' + d)
            self.logtxt.appendPlainText('Error: ' + d)

    @pyqtSlot()
    def on_btnOK_clicked(self):
        """
        Download starten
        """
        # progressbar und Logmeldungen löschen
        self.progressBar.value = 0
        self.logtxt.clear()
    
        # Download ausführen
        if self.edtPath.text() != '':
            self.y_download(self.edtURL.text())
            print('Download beendet.')
            self.logtxt.appendPlainText('Download beendet.')
        else:
            QMessageBox.critical(self, 'Fehler', 'Bitte Download-Pfad angeben.')
            return
         
    @pyqtSlot()
    def on_btnClose_clicked(self):
        """
        Programm beenden
        """
        self.close()
    
    @pyqtSlot()
    def on_btnFileDialog_clicked(self):
        """
        Download-Ordner auswählen
        """
        dialog_filter = "Alle Ordner (*)"
        directory = os.path.expanduser('~')
        dialog = QFileDialog(self, 'Ordner auswählen', directory, dialog_filter)
        dialog.setFileMode(QFileDialog.DirectoryOnly)
        myFolder = ''
        if dialog.exec_() == QDialog.Accepted:
            myFolder = dialog.selectedFiles()[0]

        if myFolder != '':
            print('Download Ordner: ' + myFolder)
            self.edtPath.setText(myFolder)

    @pyqtSlot()
    def on_btnPasteClipboard_clicked(self):
        """
        URL aus Zwischenablage einfügen
        """
        clipboard = QApplication.clipboard()
        if clipboard.mimeData().hasText():
            self.edtURL.setText(clipboard.text())
